@@ -1,6 +1,6 @@
 /**
  * A small utility class.
- * TODO: use reosurce bundle
+ * TODO: use resource bundle
  */
 class Utility {
   
@@ -17,7 +17,7 @@ class Utility {
   public static final int E_UNMATCHED = 3; 
 
   public static void error(int code) {
-	  System.out.println(errorMsg[code]);
+	  System.err.println(errorMsg[code]);
   }
 }
 
