@@ -1,7 +1,7 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  * JFlex ZeroReader example                                                *
  *                                                                         *
- * Copyright (C) 1998-2015  Gerwin Klein <lsf@jflex.de>                    *
+ * Copyright (C) 1998-2018  Gerwin Klein <lsf@jflex.de>                    *
  * All rights reserved.                                                    *
  *                                                                         *
  * License: BSD                                                            *
@@ -16,7 +16,7 @@ import java.io.IOException;
 /**
  * Reader that returns 0 chars read every once in a while.
  * 
- * This is a demonstration of a problematics Reader that does not
+ * This is a demonstration of a problematic Reader that does not
  * implement the Reader specification correctly. Do not use.
  */
 public class FunkyReader extends Reader {
@@ -31,7 +31,7 @@ public class FunkyReader extends Reader {
   public int read(char[] cbuf, int off, int len) throws IOException {
     if (!do_zero) {
       do_zero = true;
-      return reader.read(cbuf,off,Integer.min(10,len));
+      return reader.read(cbuf,off,Math.min(10,len));
     }
     else {
       do_zero = false;
